/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _CLIPCODE_NDC_H_
#define _CLIPCODE_NDC_H_	1

#ifdef __SPU__
#include <spu_intrinsics.h>
#else /* not __SPU__ */
#include <altivec.h>
#endif
/*
 * FUNCTION
 * 	unsigned int _clipcode_ndc(vector float v)
 * 
 * DESCRIPTION
 *	_clipcode_ndc generates a clip code for the normalized
 *	device coordinate v. The ndc vecto is packed in a 128-bit 
 *	floating-point vector as follows.
 *           _______________________________
 *          |___x___|___y___|___z___|___w___|
 *        
 *	The clipcode is a set of bit flags indicating if the vertex
 *	is outside the half-space defined by the -1.0 to 1.0 volume.
 *	Defines for each of the 6 bit-flags are defined in libvertex.h
 *
 *	The clipcode is logically computed with the folowing pseduo-code:
 *
 *	  code = 0;
 *	  if (x < -w) code |= CLIP_CODE_LEFT;
 *	  if (x >  w) code |= CLIP_CODE_RIGHT;
 *	  if (y < -w) code |= CLIP_CODE_BOTTOM;
 *	  if (y >  w) code |= CLIP_CODE_TOP;
 *	  if (z < -w) code |= CLIP_CODE_NEAR;
 *	  if (z >  w) code |= CLIP_CODE_FAR;
 *
 *	Due to the implementation, the least significant
 *	bits of the clipcode are also set if the w is negative.
 *	This is an indication that the vertex is behind the
 *	viewer.
 */

static __inline unsigned int _clipcode_ndc(vector float v)
{
  vector float w;
#ifdef __SPU__
  vector float minus_w;
  vector unsigned int neg, pos;
  vector unsigned int selector = spu_splats((unsigned int)0xFFFF0000);

  w = spu_shuffle(v, v, ((vector unsigned char) {
				    12, 13, 14, 15, 12, 13, 14, 15,
				    12, 13, 14, 15, 12, 13, 14, 15}));
  minus_w = (vector float)spu_xor((vector unsigned int)w, spu_splats((unsigned int)0x80000000));

  pos = spu_cmpgt(v, w);
  neg = spu_cmpgt(minus_w, v);

  return (spu_extract(spu_gather((vector unsigned short)spu_sel(neg, pos, selector)), 0));
#else
  union {
    vector signed int iv;
    vector unsigned int uiv;
    unsigned int ui[4];
  } code;

  w = vec_splat(v, 3);
  code.uiv = vec_sr((vector unsigned int)vec_cmpb(v, w), ((vector unsigned int) { 24, 26, 28, 30}));
  code.iv = vec_sums(code.iv, ((vector signed int) {0,0,0,0}));

  return (code.ui[3]);
#endif
}

#endif /* _CLIPCODE_NDC_H_ */
