/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _LOAD_VEC_UNALIGNED_H_
#define _LOAD_VEC_UNALIGNED_H_

#ifdef __SPU__
#include <spu_intrinsics.h>
#else /* not __SPU__ */
#include <altivec.h>
#endif

/*
 * FUNCTION
 *	vector unsigned char _load_vec_unaligned(unsigned char * ptr)
 *
 * DESCRIPTION
 *	_load_vec_unaligned fetches the quadword beginning at the 
 *	address specified by the parameter ptr and returns it as a 
 *	unsigned character vector. This routine assumes that ptr
 *	is not aligned to a quadword and therefore fetches the quadword
 *	containing the byte pointed to by ptr and the following 
 *	quadword.
 *
 */

static __inline vector unsigned char _load_vec_unaligned(unsigned char * ptr)
{
  vector unsigned char qw0, qw1, qw;

#ifdef __SPU__
  unsigned int shift;
  
  shift = (unsigned int)(ptr) & 15;
  qw0 = *((vector unsigned char *)(ptr));
  qw1 = *((vector unsigned char *)(ptr+16));

  qw = spu_or(spu_slqwbyte(qw0, shift), spu_rlmaskqwbyte(qw1, (signed int)(shift-16)));

#else

  qw0 = *((vector unsigned char *)(ptr));
  qw1 = *((vector unsigned char *)(ptr+16));

  qw = vec_perm(qw0, qw1, vec_lvsl(0, ptr));

#endif /* __SPU__ */
  return (qw);
}

#endif /*  _LOAD_VEC_UNALIGNED_H_ */



