/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _CAST_MATRIX_4X4_TO_FLT_H_
#define _CAST_MATRIX_4X4_TO_FLT_H_	1

#ifdef __SPU__
#include <spu_intrinsics.h>
#else /* not __SPU__ */
#include <altivec.h>
#endif

/*
 * FUNCTION
 * 	void _cast_matrix4x4_to_flt(vector float *mOut, vector double *mIn)
 * 
 * DESCRIPTION
 *	_cast_matrix4x4_to_flt converts a 4x4 double precision floating-point
 *	matrix into a single precision matrix. The input and output matrix
 *	are pointed to by mIn and mOut respectively and are 128-bit aligned.
 */
#ifdef __SPU__

static __inline void _cast_matrix4x4_to_flt(vector float *mOut, const vector double *mIn)
{
  vector unsigned char shuffle = (vector unsigned char) {
					     0, 1, 2, 3,  8, 9,10,11, 
					     16,17,18,19, 24,25,26,27};

  *(mOut + 0) = spu_shuffle(spu_roundtf(*(mIn + 0)), spu_roundtf(*(mIn + 1)), shuffle);
  *(mOut + 1) = spu_shuffle(spu_roundtf(*(mIn + 2)), spu_roundtf(*(mIn + 3)), shuffle);
  *(mOut + 2) = spu_shuffle(spu_roundtf(*(mIn + 4)), spu_roundtf(*(mIn + 5)), shuffle);
  *(mOut + 3) = spu_shuffle(spu_roundtf(*(mIn + 6)), spu_roundtf(*(mIn + 7)), shuffle);
}

#endif /* __SPU__ */
#endif /* _CAST_MATRIX_4X4_TO_FLT_H_ */
