/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2006,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */

#ifndef _MPM_NEG_H_
#define _MPM_NEG_H_	1

#include <mpm_defines.h>

/* mpm_neg
 * -------
 * Generic routine that negates the large number pointed to by <a> of <size>
 * quadwords. 
 *	<n> = -<a>
 *
 * Note: <n> can be the same array as <a>, but they can not overlap.
 */
static __inline void _mpm_neg(vector unsigned int *n, const vector unsigned int *a, int size)
{
  int i;
  vector unsigned int ni, ai, no_carry, addend;

  /* Invert all the bits and add 1. Terminate loop
   * if addition propogation terminates.
   */
  for (i=size-1; i>=0; ) {
    ai = a[i];
    ni = spu_xor(ai, -1);
    no_carry = spu_xor(spu_cmpeq(ai, 0), -1);

    addend = spu_xor(spu_or(spu_slqwbyte(no_carry, 12),
			    spu_or(spu_slqwbyte(no_carry, 8),
				   spu_slqwbyte(no_carry, 4))), -1);
    n[i--] = spu_sub(ni, addend);

    if (spu_extract(spu_gather(no_carry), 0)) break;		 
  }
  
  /* Finish inverting any remaining most significant quadwords.
   */
  for (; i>=0; i--) n[i] = spu_xor(a[i], -1);
}

#endif /* _MPM_NEG_H_ */
