/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <spu_intrinsics.h>
#include "mpm_macros.h"

#ifndef _MPM_SUB2_H_
#define _MPM_SUB2_H_	1


/* mpm_sub2
 * --------
 * Generic routine that subtracts two large numbers pointed to by <a> and <b> 
 * of <asize> and <bsize> quadwords, respectively. 
 *
 * <a> must be larger or equal to <b>, however, <asize> can be smaller than
 * <bsize>. The result is returned in memory pointed to by <s> and is <asize>
 * quadwords in length.
 *
 *	<s> = <a> - <b>
 */
static __inline void _mpm_sub2(vector unsigned int *s, const vector unsigned int *a, int asize, const vector unsigned int *b, int bsize)
{
  int minsize;
  vector unsigned int borrow, selector;

  /* Make sure that the size (number of quadwords) of a is
   * at least as big as b. Since a must be bigger than b, we
   * skip leading quadwords of b if bsize > asize;
   */
  selector = spu_cmpgt(spu_promote(asize, 0), spu_promote(bsize, 0));
  minsize = MPM_SELECT(asize, bsize, selector, int);

  borrow = spu_splats((unsigned int)1);
  while (minsize--) {
    asize--;
    bsize--;
    MPM_SUB_FULL(s[asize], borrow, a[asize], b[bsize], borrow)
  }
  
  while (asize--) {
    MPM_SUB_FULL(s[asize], borrow, a[asize], spu_splats((unsigned int)0), borrow)
  }
}

#endif /* _MPM_SUB2_H_ */
