/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _UNPACK_COLOR8_H_
#define _UNPACK_COLOR8_H_	1

#ifdef __SPU__
#include <spu_intrinsics.h>
#else /* not __SPU__ */
#include <altivec.h>
#endif

/*
 * FUNCTION
 *	vector float _unpack_color8(unsigned int color);
 * 
 * DESCRIPTION
 *	_unpack_color8 converts a 32-bit unsigned integer consisting of 4
 *	8-bit packed color components and produces a vectored floating-point
 *	normalized color in which each channel of the vectored color is 
 *	a seperate channel.
 */

static __inline vector float _unpack_color8(unsigned int color)
{
  vector float result;
#ifdef __SPU__
  vector unsigned int rgba;

  vector unsigned char shuffle = (vector unsigned char) {
					     0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01,
					     0x02, 0x02, 0x02, 0x02, 0x03, 0x03, 0x03, 0x03};
  rgba = spu_promote(color, 0);
  rgba = spu_shuffle(rgba, rgba, shuffle);
  result = spu_convtf(rgba, 32);

#else
  union {
    unsigned int ui[4];
    vector unsigned int uiv;
  } rgba;

  vector unsigned char shuffle = (vector unsigned char) {
					     0x10, 0x00, 0x00, 0x00, 0x10, 0x01, 0x01, 0x01,
					     0x10, 0x02, 0x02, 0x02, 0x10, 0x03, 0x03, 0x03};
  vector unsigned int vzero = ((vector unsigned int) {0,0,0,0});

  rgba.ui[0] = color;
  rgba.uiv = vec_perm(rgba.uiv, vzero, shuffle);
  result = vec_ctf(rgba.uiv, 24);
#endif
  return (result);
}


#endif /* _UNPACK_COLOR8_H_ */
